import { Component, OnInit, Inject, Renderer2 } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material';
import { KEY_LOGIN_RES_INFO, KEY_VER_INFO } from '@app/types';
import {
  SessionStorageService,
  LocalStorageService
} from '@ucap-webmessenger/web-storage';

import { Store } from '@ngrx/store';

import clone from 'clone';

import {
  DialogService,
  TranslateService as UCapTranslateService,
  DateService as UCapDateService
} from '@ucap-webmessenger/ui';
import { VersionInfo2Response } from '@ucap-webmessenger/api-public';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { map } from 'rxjs/operators';
import { DOCUMENT } from '@angular/common';
import { AppUserInfo, KEY_APP_USER_INFO } from '@app/types/app-user-info.type';
import { environment } from '../../../../../environments/environment';
import {
  GeneralSetting,
  Settings,
  NotificationSetting
} from '@ucap-webmessenger/ui-settings';
import { NativeService, UCAP_NATIVE_SERVICE } from '@ucap-webmessenger/native';
import { TranslateService } from '@ngx-translate/core';
import { ObjectUtil } from '@ucap-webmessenger/core';
import { OptionProtocolService } from '@ucap-webmessenger/protocol-option';

export interface MessengerSettingsDialogData {}

export interface MessengerSettingsDialogResult {}

@Component({
  selector: 'app-messenger-settings-dialog',
  templateUrl: './messenger-settings.dialog.component.html',
  styleUrls: ['./messenger-settings.dialog.component.scss']
})
export class MessengerSettingsDialogComponent implements OnInit {
  loginRes: LoginResponse;
  sessionVerinfo: VersionInfo2Response;

  modifiedSettings: Settings;

  appUserInfo: AppUserInfo;

  constructor(
    public dialogRef: MatDialogRef<
      MessengerSettingsDialogData,
      MessengerSettingsDialogResult
    >,
    @Inject(MAT_DIALOG_DATA) public data: MessengerSettingsDialogData,
    private dialogService: DialogService,
    private sessionStorageService: SessionStorageService,
    private localStorageService: LocalStorageService,
    private ucapTranslateService: UCapTranslateService,
    private ucapDateService: UCapDateService,
    private optionProtocolService: OptionProtocolService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private translateService: TranslateService,
    private store: Store<any>,
    @Inject(DOCUMENT) private document: Document,
    private renderer2: Renderer2
  ) {
    this.appUserInfo = this.localStorageService.encGet<AppUserInfo>(
      KEY_APP_USER_INFO,
      environment.customConfig.appKey
    );

    this.modifiedSettings = clone(this.appUserInfo.settings);

    this.sessionVerinfo = this.sessionStorageService.get<VersionInfo2Response>(
      KEY_VER_INFO
    );
    this.loginRes = this.sessionStorageService.get<LoginResponse>(
      KEY_LOGIN_RES_INFO
    );
  }

  ngOnInit() {}

  onChangedGeneralSetting(setting: GeneralSetting) {
    this.applySettings(this.modifiedSettings, {
      ...this.appUserInfo.settings,
      general: setting
    });
  }

  onChangedNotificationSetting(setting: NotificationSetting) {
    this.applySettings(this.modifiedSettings, {
      ...this.appUserInfo.settings,
      notification: setting
    });
  }

  onClickChoice(choice: boolean): void {
    if (choice) {
      if (ObjectUtil.equals(this.appUserInfo.settings, this.modifiedSettings)) {
        // There is not modified settings.
        return;
      }

      this.appUserInfo.settings = this.modifiedSettings;
      this.localStorageService.encSet<AppUserInfo>(
        KEY_APP_USER_INFO,
        this.appUserInfo,
        environment.customConfig.appKey
      );
    } else {
      this.applySettings(this.modifiedSettings, this.appUserInfo.settings);
    }

    this.dialogRef.close({});
  }

  private applySettings(oriSettings: Settings, modSettings: Settings) {
    if (oriSettings.general.appTheme !== modSettings.general.appTheme) {
      this.renderer2.setAttribute(
        this.document.body,
        'class',
        modSettings.general.appTheme
      );
    }
    if (oriSettings.general.locale !== modSettings.general.locale) {
      this.translateService.use(modSettings.general.locale);
    }
    if (oriSettings.general.hrInfoLocale !== modSettings.general.hrInfoLocale) {
      this.ucapTranslateService.use(modSettings.general.hrInfoLocale);
    }

    if (oriSettings.general.timezone !== modSettings.general.timezone) {
      this.ucapDateService.use(modSettings.general.timezone);
    }

    if (oriSettings.general.autoLaunch !== modSettings.general.autoLaunch) {
      this.nativeService.changeAutoLaunch(modSettings.general.autoLaunch);
    }
    if (
      oriSettings.general.startupHideWindow !==
      modSettings.general.startupHideWindow
    ) {
      this.nativeService.changeStartupHideWindow(
        modSettings.general.startupHideWindow
      );
    }

    oriSettings = modSettings;
  }
}
